import io

from dli.csvparser.dialect import Dialect


class CsvParser:
    def __init__(self, file, dialect=None):
        self.dialect = dialect or Dialect()
        self.buffer = file
        self.done = False

    def _read_next_char(self):
       return self.buffer.read(1)[:1]

    def _peek_next_chars(self):
        return self.buffer.peek(1)[:1]

    def _read_quoted_field(self):
        # skip first quote_char
        self._read_next_char()
        tmp_buf = []
        last_char = None
        while True:
            char = self._read_next_char()
            if char != self.dialect.quote_char:
                # Normal character or escape handling
                if last_char is None or last_char != self.dialect.escape_char:
                    tmp_buf.append(char)
                    last_char = char
                else:
                    last_char = None
            else:
                # Now, check if we have a closing quote or an escaped quote
                if not tmp_buf:
                    # If the buffer is empty, we are at the end of the quoted field
                    return b''.join(tmp_buf).decode("utf-8")

                # Check the last character for an escape sequence
                if last_char == self.dialect.escape_char:
                    # If it's an escape, it's part of the field
                    tmp_buf[-1] = char
                    last_char = None
                else:
                    # Otherwise, it's the end of the quoted field
                    return b''.join(tmp_buf).decode("utf-8")

    def _read_unquoted_field(self):
        """Reads an unquoted field."""
        s = []  # Use the tmp_buf to accumulate the field
        while True:
            char = self._peek_next_chars()
            if not char:
                return b''.join(s).decode("utf-8")
            if char in (self.dialect.delimiter, self.dialect.line_terminator):
                return b''.join(s).decode("utf-8")
            s.append(self._read_next_char())

    def read(self):
        record = []

        while True:
            char = self._peek_next_chars()
            if not char:
                return None

            if char == self.dialect.quote_char:
                field = self._read_quoted_field()
            else:
                field = self._read_unquoted_field()

            record.append(field)

            # Check if the next character is the delimiter or line terminator
            next_char = self._peek_next_chars()
            if next_char == self.dialect.line_terminator:
                self._read_next_char()  # Skip the line terminator
                break
            elif next_char == self.dialect.delimiter:
                self._read_next_char()  # Skip the delimiter
            elif not next_char:
                break

        return record

    # Iterator Methods
    def __iter__(self):
        return self

    def __next__(self):
        if self.done:
            raise StopIteration
        record = self.read()
        if not record:
            self.done = True
            raise StopIteration
        return record