#!/usr/bin/python
# -*- coding: UTF-8 -*-

"""
    Huawei DLI Python SDK
    The DliClient class of DLI Python SDK
"""

import time

from huaweicloudsdkcore.exceptions import exceptions
from huaweicloudsdkdli.v1.dli_client import DliClient as InnerDliClient
from typing_extensions import deprecated

from dli.dli_info import DliInfo
from dli.exception import DliException
from dli.job import SparkBatchJob
from dli.job import SqlJob
from dli.logger import logger
from dli.queue import Queue


class DliClient(object):
    MAX_PAGE_SIZE = 100

    def __init__(self, **kwargs):
        """
        Init a DLI client

        :param str region: The region, for example, cn-north-1
        :param str project_id: On the management console, displayed My Credential page,
                                click Project List to query the project ID,
                                for example, 5a3314075bfa49b9ae360f4ecd333695.
        :param str domain: domain, default: myhuaweicloud.com
        :param str ak: Access Key
        :param str sk: Secret Key
        :param str proxy: proxy address, for example: https://192.168.64.187:3128
        """

        self._sdk_version = '2.0.3'
        self._dli_info = DliInfo(**kwargs)
        from huaweicloudsdkcore.auth.credentials import BasicCredentials
        self._credentials = BasicCredentials(self._dli_info.ak, self._dli_info.sk, self._dli_info.project_id)
        if self._dli_info.security_token is not None:
            self._credentials.security_token = self._dli_info.security_token
        self._inner_client = InnerDliClient.new_builder() \
            .with_credentials(self._credentials) \
            .with_endpoints([self.dli_info.dli]) \
            .build()

    @property
    def sdk_version(self):
        return self._sdk_version

    @property
    def dli_info(self):
        return self._dli_info

    @property
    def inner_client(self):
        return self._inner_client

    @inner_client.setter
    def inner_client(self, inner_client):
        self._inner_client = inner_client

    @property
    def rest_api(self):
        raise DliException("Unsupported")

    @property
    def token_expires_time(self):
        raise DliException("Unsupported")

    @property
    def token(self):
        raise DliException("Unsupported")

    def get_token(self):
        raise DliException("Unsupported")

    def is_token_expired(self):
        raise DliException("Unsupported")

    @deprecated("Please use huaweicloudsdkdli directly, this method will be remove soon!")
    def list_queues(self):
        """
        This method is used to list all queues under the project.

        :return: list
                The list of all queues.
        :raises: DliException
        """

        try:
            from huaweicloudsdkdli.v1 import ListQueuesRequest
            resp = self.inner_client.list_queues_invoker(ListQueuesRequest(queue_type="all")) \
                .add_header("X-User-Agent", "dli-sdk-python/" + self.sdk_version).invoke()
        except exceptions.ClientRequestException as e:
            raise DliException(api_exception=e)
        if not resp.is_success:
            raise DliException(msg=resp.message)

        queues = []
        for v1_queue in resp.queues:
            # huaweicloudsdkdli.v1.Queue -> Queue
            queues.append(Queue(v1_queue))
        return queues

    @deprecated("Please use huaweicloudsdkdli directly, this method will be remove soon!")
    def get_queue(self, queue_name):
        """
        This method is used to get a queue by queue name.

        :param str queue_name: Queue name
        :return: Queue
                If the Queue do not exist, will return None
        :raises: DliException
        """

        try:
            from huaweicloudsdkdli.v1 import ShowQueueRequest
            resp = self.inner_client.show_queue_invoker(ShowQueueRequest(queue_name)) \
                .add_header("X-User-Agent", "dli-sdk-python/" + self.sdk_version).invoke()
        except exceptions.ClientRequestException as e:
            raise DliException(api_exception=e)
        if not resp.is_success:
            raise DliException(msg=resp.message)

        # ShowQueueResponse -> Queue
        return Queue(resp)

    def _submit_sql_job(self, sql, db_name=None, queue_name='default', catalog='dli', engine='spark', options=None):
        conf_list = ['dli.sql.sqlasync.enabled=true']
        if options:
            for key, value in options.items():
                conf_list.append(key + '=' + value)

        try:
            from huaweicloudsdkdli.v1 import CreateSqlJobRequest, CreateSqlJobRequestBody
            resp = self.inner_client.create_sql_job_invoker(CreateSqlJobRequest(CreateSqlJobRequestBody(
                currentdb=db_name, sql=sql, queue_name=queue_name, conf=conf_list, current_catalog=catalog,
                engine_type=engine))) \
                .add_header("X-User-Agent", "dli-sdk-python/" + self.sdk_version).invoke()
        except exceptions.ClientRequestException as e:
            raise DliException(api_exception=e)
        if not resp.is_success:
            raise DliException(msg=resp.message)

        return resp

    def async_execute_sql(self, sql, db_name=None, queue_name='default', catalog='dli', engine='spark', options=None):
        """
        Execute SQL job in asynchronous mode.
        This method is used to submit SQL statements to the server cluster.
        The job contains the following types: DDL, DCL, IMPORT, EXPORT, QUERY and INSERT.

        :param str sql: SQL statement
        :param str queue_name: Queue name for the SQL execute, use 'default' queue for default
        :param str db_name: Database name of the table belongs to, if the SQL statement have
                             database name, it can set to None.
        :param dict options: User-defined parameter that applies to the job
                             examples:
                             {'dli.sql.cbo.enabled': True,
                              'dli.sql.syncMode.enabled': True}
        :return: SqlJob
        """

        resp = self._submit_sql_job(sql, db_name, queue_name, catalog, engine, options)
        return SqlJob(resp.job_id, resp.job_type, self)

    def execute_sql(self, sql, db_name=None, queue_name='default', catalog='dli', engine='spark', options=None,
                    retry_times=1000):
        """
        Execute SQL job
        This method is used to submit SQL statements to the server cluster.
        The job contains the following types: DDL, DCL, IMPORT, EXPORT, QUERY and INSERT.
        This method is a synchronous mode, blocking until SQL execution is completed.

        :param str sql: SQL statement
        :param str queue_name: Queue name for the SQL execute, use 'default' queue for default
        :param str db_name: Database name of the table belongs to, if the SQL statement have
                             database name, it can set to None.
        :param dict options: User-defined parameter that applies to the job
                             examples:
                             {'dli.sql.cbo.enabled': True,
                              'dli.sql.syncMode.enabled': True}
        :return: SqlJob
        """

        resp = self._submit_sql_job(sql, db_name, queue_name, catalog, engine, options)
        sql_job = SqlJob(resp.job_id, resp.job_type, self)
        self._cycle_check_sql_job(sql_job, retry_times)
        return sql_job

    @deprecated("Please use huaweicloudsdkdli directly, this method will be remove soon!")
    def cancel_sql(self, job_id):
        """

        :param str job_id: ID of a job returned after a job is submitted.
        :return:
        """

        try:
            if not job_id:
                raise DliException(msg="No job in progress, cancel job failed")
            from huaweicloudsdkdli.v1 import CancelSqlJobRequest
            resp = self.inner_client.cancel_sql_job_invoker(CancelSqlJobRequest(job_id)) \
                .add_header("X-User-Agent", "dli-sdk-python/" + self.sdk_version).invoke()
        except exceptions.ClientRequestException as e:
            raise DliException(api_exception=e)
        if not resp.is_success:
            raise DliException(msg=resp.message)

    @deprecated("Please use huaweicloudsdkdli directly, this method will be remove soon!")
    def list_sql_jobs(self):
        """
        This method is used to list all sql jobs under the project.

        :return: list
                The list of all jobs.
        :raises: DliException
        """

        sql_jobs = []
        job_info_size = 101
        i = 1
        while job_info_size >= 100:
            job_infos = self.list_sql_jobs_page(100, i)
            job_info_size = len(job_infos)
            i += 1
            sql_jobs = sql_jobs + job_infos
        return sql_jobs

    @deprecated("Please use huaweicloudsdkdli directly, this method will be remove soon!")
    def list_sql_jobs_page(self, page_size, current_page):
        """
        This method is used to list sql jobs by page under the project.

        :return: list
                The list of all jobs.
        :raises: DliException
        """

        try:
            from huaweicloudsdkdli.v1 import ListSqlJobsRequest
            resp = self.inner_client.list_sql_jobs_invoker(ListSqlJobsRequest(
                current_page=current_page, page_size=page_size)) \
                .add_header("X-User-Agent", "dli-sdk-python/" + self.sdk_version).invoke()
        except exceptions.ClientRequestException as e:
            raise DliException(api_exception=e)
        if not resp.is_success:
            raise DliException(msg=resp.message)

        sql_jobs = []
        for v1_sql_job in resp.jobs:
            # huaweicloudsdkdli.v1.SqlJob
            sql_jobs.append(v1_sql_job)
        return sql_jobs

    def _cycle_check_sql_job(self, sql_job, retry_times=1000):
        loop_count = 0
        while True:
            if loop_count < 20:
                time.sleep(0.5)
            elif loop_count < 50:
                time.sleep(1)
            else:
                time.sleep(3)

            resp_status, resp_msg = sql_job.get_status()
            logger.info("current status is: %s", resp_status)
            if resp_status == 'FAILED':
                message = 'Get job status, Job run failed.\n' \
                          'Job id = {0}\nDLI response: {1}'.format(sql_job.job_id, resp_msg)
                raise DliException(msg=message)
            elif resp_status == 'FINISHED' or resp_status == 'CANCELLED':
                break

            # The sql is running too long times
            if loop_count > retry_times:
                message = 'Job run too long time, Job id = ' + sql_job.job_id
                raise DliException(msg=message)

            loop_count += 1
        return resp_status

    @deprecated("Please use huaweicloudsdkdli directly, this method will be remove soon!")
    def wait_for_finish(self, job_id, retry_times=1000):
        """

        :param str job_id: ID of a job returned after a job is submitted.
        :param int retry_times: The retry interval
        :return: str job_status: The status of this job.
        """

        loop_count = 0
        while True:
            if loop_count < 100:
                time.sleep(0.5)
            else:
                time.sleep(3)

            try:
                from huaweicloudsdkdli.v1 import ShowSqlJobStatusRequest
                resp = self.inner_client.show_sql_job_status_invoker(ShowSqlJobStatusRequest(job_id)) \
                    .add_header("X-User-Agent", "dli-sdk-python/" + self.sdk_version).invoke()
            except exceptions.ClientRequestException as e:
                raise DliException(api_exception=e)
            if not resp.is_success:
                raise DliException(msg=resp.message)

            if resp.status == 'FAILED':
                message = 'Get job status, Job run failed.\n' \
                          'Job id = {0}\nDLI response: {1}'.format(resp.job_id, resp.message)
                raise DliException(msg=message)
            elif resp.status == 'FINISHED' or resp.status == 'CANCELLED':
                break

            # The sql is running too long times
            if loop_count > retry_times:
                message = 'Job run too long time, Job id = ' + resp.job_id
                raise DliException(msg=message)

            loop_count += 1

        return resp.status

    @deprecated("Please use huaweicloudsdkdli directly, this method will be remove soon!")
    def export_table(self, table_name, db_name, data_path, queue_name='default', compress='none',
                     export_mode='ErrorIfExists'):
        """
        This method is used to export data to a file. You can export data from DLI
        tables to files. Currently, data can be exported only to OBS, and the OBS
        path must be specified to the folder level.
        This method is synchronous, blocking until execution is completed.

        :param str table_name: Name of the table from which data is exported.
        :param str db_name: Name of the database where the table from.
        :param str data_path: Path for storing the exported data (currently,
                            data can be stored on an OBS path starting with s3a only).
        :param str queue_name: Name of the queue that is specified to execute a task.
        :param str compress: Compression mode for exported data. Currently, the
                            compression modes gzip, bzip2, and deflate are supported. If you
                            do not want to compress data, enter none.
        :param str export_mode: Export mode. The parameter value can be ErrorIfExists
                            or Overwrite. If export_mode is not specified, this parameter
                            is set to ErrorIfExists by default.
        :return: tuple (job_id, status)
                    str job_id: ID of a job returned after a job is submitted.
                    str status: The status of this job.
        """

        try:
            from huaweicloudsdkdli.v1 import ExportTableRequest, ExportTableRequestBody
            resp = self.inner_client.export_table_invoker(ExportTableRequest(ExportTableRequestBody(
                data_path=data_path, data_type='csv', compress=compress, queue_name=queue_name,
                database_name=db_name, table_name=table_name, export_mode=export_mode))) \
                .add_header("X-User-Agent", "dli-sdk-python/" + self.sdk_version).invoke()
        except exceptions.ClientRequestException as e:
            raise DliException(api_exception=e)
        if not resp.is_success:
            raise DliException(msg=resp.message)

        status = self._cycle_check_sql_job(SqlJob(resp.job_id, "UnUsed", self))
        return resp.job_id, status

    @deprecated("Please use huaweicloudsdkdli directly, this method will be remove soon!")
    def import_table(self, table_name, db_name, data_path, data_type, queue_name='default', options=None):
        """
        This method is used to import data from a file to a DLI table. Currently,
        only OBS data can be imported to DLI.
        This method is synchronous, blocking until execution is completed.

        :param str table_name: Name of the table to which data is imported.
        :param str db_name: Name of the database where the table to which data
                            is imported resides.
        :param str data_path: Path to which data is imported. Currently, only data
                            on an OBS path starting with s3a can be imported.
        :param str data_type: Type of the data to be imported. Currently, data types
                            CSV, Parquet, ORC, and JSON are supported.
        :param str queue_name: Name of the queue that is specified to execute a task.
                            If no queue is specified, the default queue is used.
        :param dict options: The dict options of the data type, for the detailed information refer to
                            https://support.huaweicloud.com/api-uquery/uquery_02_0100.html
                            For examples:
                            {
                                "with_column_header": true,
                                "delimiter": ",",
                                "quote_char": "\"",
                                "escape_char": "\\",
                                "date_format": "yyyy/MM/dd",
                                "timestamp_format": "yyyy/MM/dd hh:mm:ss"
                            }
        :return: tuple (job_id, status)
                    str job_id: ID of a job returned after a job is submitted.
                    str status: The status of this job.
        """

        delimiter = None
        quote_char = None
        escape_char = None
        date_format = None
        timestamp_format = None
        with_column_header = None

        if options:
            delimiter = options.get('delimiter')
            quote_char = options.get('quote_char')
            escape_char = options.get('escape_char')
            date_format = options.get('date_format')
            timestamp_format = options.get('timestamp_format')
            with_column_header = options.get('with_column_header')
        from huaweicloudsdkdli.v1 import ImportTableRequestBody
        body = ImportTableRequestBody(data_path=data_path, data_type=data_type,
                                      queue_name=queue_name, database_name=db_name, table_name=table_name,
                                      with_column_header=with_column_header,
                                      delimiter=delimiter, quote_char=quote_char, escape_char=escape_char,
                                      date_format=date_format, timestamp_format=timestamp_format)

        try:
            from huaweicloudsdkdli.v1 import ImportTableRequest
            resp = self.inner_client.import_table_invoker(ImportTableRequest(body)) \
                .add_header("X-User-Agent", "dli-sdk-python/" + self.sdk_version).invoke()
        except exceptions.ClientRequestException as e:
            raise DliException(api_exception=e)
        if not resp.is_success:
            raise DliException(msg=resp.message)

        status = self._cycle_check_sql_job(SqlJob(resp.job_id, "Unused", self))
        return resp.job_id, status

    @deprecated("Please use huaweicloudsdkdli directly, this method will be remove soon!")
    def submit_spark_batch_job(self, queue_name, batch_job_info):
        """

        Submit spark batch job
        This method is used to submit spark batch job to the server cluster.
        This method is a synchronous mode, blocking until spark batch job is completed.

        :param str queue_name:
        :param SparkJobInfo batch_job_info:
        :return: SparkBatchJob
        """

        body_dict = batch_job_info.get_request_body(queue_name)

        try:
            from huaweicloudsdkdli.v1 import CreateSparkJobRequest, CreateSparkJobRequestBody
            resp = self.inner_client.create_spark_job_invoker(CreateSparkJobRequest(body=CreateSparkJobRequestBody(
                file=body_dict['file'], class_name=body_dict['className'], queue=queue_name,
                args=body_dict['args'], sc_type=body_dict['sc_type'], jars=body_dict['jars'],
                py_files=body_dict['pyFiles'], files=body_dict['files'], conf=body_dict['conf'],
                resources=body_dict['resources'], groups=body_dict['groups'], name=body_dict['name'],
                modules=body_dict['modules'], catalog_name=body_dict['catalog_name'],
                driver_memory=body_dict['driver_memory'], driver_cores=body_dict['driver_cores'],
                executor_memory=body_dict['executor_memory'], executor_cores=body_dict['executor_cores'],
                num_executors=body_dict['num_executors'], obs_bucket=body_dict['obs_bucket'],
                auto_recovery=body_dict['auto_recovery'], max_retry_times=body_dict['max_retry_times'],
                feature=body_dict['feature'], image=body_dict['image']))) \
                .add_header("X-User-Agent", "dli-sdk-python/" + self.sdk_version).invoke()
        except exceptions.ClientRequestException as e:
            raise DliException(api_exception=e)

        return SparkBatchJob(self, resp.id)

    @deprecated("Please use huaweicloudsdkdli directly, this method will be remove soon!")
    def del_spark_batch_job(self, batch_id):
        """

        :param str batch_id: ID of a spark batch job returned after a spark batch job is submitted.
        :return:
        """

        try:
            if not batch_id:
                raise DliException(msg="No spark batch job in progress, cancel spark batch job failed")
            from huaweicloudsdkdli.v1 import CancelSparkJobRequest
            resp = self.inner_client.cancel_spark_job_invoker(CancelSparkJobRequest(batch_id)) \
                .add_header("X-User-Agent", "dli-sdk-python/" + self.sdk_version).invoke()
        except exceptions.ClientRequestException as e:
            raise DliException(api_exception=e)
        return resp.msg

    def set_verify_ssl(self, verify_ssl):
        raise DliException("Unsupported")
