from dli.table import TableSchema

from dli.logger import logger
from dli.sql_engine import SqlEngine
from dli.table_desc_util import parse_spark_show_table_result, parse_hetu_show_table_result, create_table_sql
from dli.table_storage_type import TableStorageTypes


class DliSqlClient(object):

    def __init__(self, dli_client):
        self._dli_client = dli_client

    def query(self, sql, database='default', queue='default', catalog='dli', engine=SqlEngine.SPARK.value, options=None,
              retry_times=1000):
        logger.info("Start exec query sql")
        sql_job = self._dli_client.execute_sql(sql, database, queue, catalog, engine, options, retry_times)
        logger.info("Exec query sql success:" + sql_job._job_id)
        result_set = sql_job.get_result(queue_name=queue)
        rows = []
        while result_set.next():
            rows.append(result_set.fetch().columns)
        return result_set._table_schema, rows

    def query_rows(self, sql, database='default', queue='default', catalog='dli', engine=SqlEngine.SPARK.value,
                   options=None,
                   retry_times=1000):
        _, rows = self.query(sql, database=database, queue=queue, catalog=catalog, engine=engine, options=options,
                             retry_times=retry_times)
        return rows

    def list_databases(self, database_pattern=None, queue='default', catalog=None, engine=SqlEngine.SPARK.value):
        """
        This method is used to list database by filter, just spark engine support
        database_pattern: '*filter_dn_name*'
        """
        sql = "SHOW DATABASES"
        if SqlEngine.SPARK.value == engine:
            if catalog is not None and len(catalog) > 0:
                sql += ' FROM ' + catalog
            if database_pattern is not None and len(database_pattern) > 0:
                sql += ' LIKE \'' + database_pattern + '\''
        elif SqlEngine.HETU.value == engine:
            if catalog is not None and len(catalog) > 0:
                sql += ' FROM "' + catalog + '"'
            if database_pattern is not None and len(database_pattern) > 0:
                sql += ' LIKE \'' + database_pattern + '\''
        rows = self.query_rows(sql, queue=queue, catalog=catalog, engine=engine)
        return [row[0] for row in rows]

    def database_exists(self, database, queue='default', catalog=None, engine=SqlEngine.SPARK.value):
        dbs = self.list_databases(database, queue, catalog, engine)
        if dbs is None or len(dbs) == 0:
            return False
        if database in dbs:
            return True
        return False

    def list_tables(self, database, tbl_pattern=None, queue='default', catalog=None, engine=SqlEngine.SPARK.value):
        sql = "SHOW TABLES FROM "
        if SqlEngine.SPARK.value == engine:
            sql += '`' + database + '`'
        elif SqlEngine.HETU.value == engine:
            sql += '"' + database + '"'
        else:
            sql += database
        if tbl_pattern is not None and len(tbl_pattern) > 0:
            sql += " LIKE '{}'".format(tbl_pattern)
        rows = self.query_rows(sql, database, queue, catalog, engine)
        if SqlEngine.SPARK.value == engine:
            return [row[1] for row in rows]
        else:
            return [row[0] for row in rows]

    def table_exists(self, database, table, queue='default', catalog=None, engine=SqlEngine.SPARK.value):
        tbls = self.list_tables(database, table, queue, catalog, engine)
        if tbls is None or len(tbls) == 0:
            return False
        if table in tbls:
            return True
        return False

    def get_table(self, database, table, queue='default', catalog=None, engine=SqlEngine.SPARK.value):
        sql = "DESCRIBE EXTENDED `{}`.`{}`".format(database, table)
        if SqlEngine.HETU.value == engine:
            sql = "DESCRIBE \"{}\".\"{}\"".format(database, table)
        rows = self.query_rows(sql, database, queue, catalog, engine)
        if SqlEngine.SPARK.value == engine:
            return parse_spark_show_table_result(database, table, rows)
        else:
            return parse_hetu_show_table_result(database, table, rows)

    def get_table_schema(self, database, table, queue='default', catalog=None, engine=SqlEngine.SPARK.value):
        tbl = self.get_table(database, table, queue, catalog, engine)
        return TableSchema(table, tbl.columns, [])

    def create_database(self, database, queue='default', catalog=None):
        """
        This method is used to create database, just spark engine support
        """

        sql = "CREATE DATABASE IF NOT EXISTS `{}`".format(database)
        logger.info("Start exec create database:" + database)
        sql_job = self._dli_client.execute_sql(sql, database, queue, catalog, SqlEngine.SPARK.value)
        logger.info("Exec create database success:" + sql_job._job_id)

    def create_hive_obs_table(
            self,
            database,
            table,
            columns,
            location,
            comment=None,
            storage_type=TableStorageTypes.PARQUET.value,
            tbl_properties=None,
            queue='default',
            catalog=None):
        """
        This method is used to create hive external table, just spark engine support
        """
        sql = create_table_sql(database, table, columns, comment, storage_type, location, tbl_properties)
        logger.info("Start exec create obs table:" + table)
        sql_job = self._dli_client.execute_sql(sql, database, queue, catalog, SqlEngine.SPARK.value)
        logger.info("Exec create table obs success:" + sql_job._job_id)

    def create_hive_dli_table(
            self,
            database,
            table,
            columns,
            comment=None,
            storage_type=TableStorageTypes.PARQUET.value,
            tbl_properties=None,
            queue='default',
            catalog=None):
        """
        This method is used to create hive dli table, just spark engine support
        """
        sql = create_table_sql(database, table, columns, comment, storage_type, tbl_properties=tbl_properties)
        logger.info("Start exec create dli table:" + table)
        sql_job = self._dli_client.execute_sql(sql, database, queue, catalog, SqlEngine.SPARK.value)
        logger.info("Exec create table dli success:" + sql_job._job_id)

    def drop_table(self, database, table, queue='default', catalog=None):
        """
        This method is used to delete table, just spark engine support
        """
        sql = "DROP TABLE IF EXISTS `{}`.`{}`".format(database, table)
        logger.info("Start exec drop table:" + table)
        sql_job = self._dli_client.execute_sql(sql, database, queue, catalog, SqlEngine.SPARK.value)
        logger.info("Exec drop table success:" + sql_job._job_id)

    def add_partition(self, database, table, partition, obs_path, queue='default', catalog=None):
        """
        This method is used to add partition, just spark engine support
        :partition: like：name='n',age=12
        """
        sql = ("ALTER TABLE `{}`.`{}` ADD PARTITION({}) LOCATION '{}'"
               .format(database, table, partition, obs_path))
        logger.info("Start exec table {} add partition {}".format(table, partition))
        sql_job = self._dli_client.execute_sql(sql, database, queue, catalog, SqlEngine.SPARK.value)
        logger.info("Exec table {} add partition {} success: {}".format(table, partition, sql_job._job_id))

    def drop_partition(self, database, table, partition, queue='default', catalog=None):
        """
        This method is used to drop partition, just spark engine support
        :partition: like：name='n',age=12
        """
        sql = ("ALTER TABLE `{}`.`{}` DROP IF EXISTS PARTITION({})".format(database, table, partition))
        logger.info("Start exec table {} drop partition {}".format(table, partition))
        sql_job = self._dli_client.execute_sql(sql, database, queue, catalog, SqlEngine.SPARK.value)
        logger.info("Exec table {} drop partition {} success: {}".format(table, partition, sql_job._job_id))

    def list_partition(self, database, table, partition_filter=None, queue='default', catalog=None,
                       engine=SqlEngine.SPARK.value):
        """
        This method is used to show partitions by filter, just spark engine support
        :partition_filter: like：name='n',age=12 or name='n',age > 12
        """
        if engine == SqlEngine.SPARK.value:
            sql = "SHOW PARTITIONS `{}`.`{}`".format(database, table)
            if partition_filter is not None and len(partition_filter) > 0:
                sql += " PARTITION({})".format(partition_filter)
            rows = self.query_rows(sql, database, queue, catalog, engine)
            return [row[0].split(":")[0].strip() for row in rows]

        if engine == SqlEngine.HETU.value:
            sql = "SHOW PARTITIONS \"{}\".\"{}\"".format(database, table)
            if partition_filter is not None and len(partition_filter) > 0:
                sql += " PARTITION({})".format(partition_filter)
            schema, rows = self.query(sql, database, queue, catalog, engine)
            result = []
            for row in rows:
                items = []
                for i in range(len(row)):
                    items.append(schema.columns[i].name + "=" + row[i])
                result.append("/".join(items))
            return result
        return []

    def list_functions(self, database, queue='default', catalog=None, engine=SqlEngine.SPARK.value):
        """
        This method is used to show functions, just spark engine support
        """
        sql = "SHOW FUNCTIONS"
        rows = self.query_rows(sql, database, queue, catalog, engine)
        return set([row[0] for row in rows])
