#!/usr/bin/python
# -*- coding: UTF-8 -*-

from datetime import datetime

from dli.csvparser.csv_parser import CsvParser
from dli.csvparser.dialect import Dialect
from dli.exception import DliException
from dli.logger import logger
from dli.row import Row

import io
import csv
import json
import os
import time


class Reader(object):
    def __init__(self, table_schema, obs_client, bucket_name, obj_prefix):
        self._table_schema = table_schema
        self._obs_client = obs_client
        self._bucket_name = bucket_name
        self._object_prefix = obj_prefix
        self._obs_objects = []
        self._list_obs_object()
        self._total_iter = len(self._obs_objects)
        self._next_iter_idx = 0
        self._current_row = None

    def __iter__(self):
        return ObsReaderIterator(self)

    def _list_obs_object(self):
        next_marker = None
        temp_files = []
        metadata_query_id = None
        logger.info("list objects on bucket: " + self._bucket_name + " with prefix: " + self._object_prefix)
        while True:
            resp = self._obs_client.listObjects(bucketName=self._bucket_name, prefix=self._object_prefix,
                                                max_keys=1000, marker=next_marker)
            for content in resp.body.contents:
                if content.key.startswith('.') or content.key.startswith('_'):
                    continue
                if content.size > 0:
                    if content.key.endswith("metadata.json"):
                        metadata_query_id = self._get_query_id_from_metadata(content.key)
                        logger.info("query id: " + metadata_query_id)
                    else:
                        logger.info("found object: " + content.key)
                        temp_files.append(content.key)
            if not resp.body.is_truncated:
                break
            next_marker = resp.body.next_marker

        for file_name in temp_files:
            if metadata_query_id is None or metadata_query_id in file_name:
                self._obs_objects.append(file_name)
        logger.info("object size: %d", len(self._obs_objects))

    def _get_query_id_from_metadata(self, metadata_file):
        try:
            local_path = "/tmp/%s" % metadata_file
            resp = self._obs_client.getObject(bucketName=self._bucket_name, objectKey=metadata_file,
                                              downloadPath=local_path)
            if resp.status < 300:
                with open(local_path, 'rb') as f:
                    metadata_json = json.loads(f.readline())
                    query_id = metadata_json.get("queryId")

                os.remove(local_path)
                return query_id
            else:
                return None
        except Exception as e:
            logger.error("Error reading metadata file: %s" % e)
            return None

    def _get_next_iter(self):
        if self._total_iter > 0 and self._next_iter_idx < self._total_iter:
            return self._create_iter()
        else:
            return None

    def _create_iter(self):
        curr_iter_idx = self._next_iter_idx
        self._next_iter_idx += 1
        object_key = self._obs_objects[curr_iter_idx]
        logger.info("create iterator for object: %s", object_key)
        resp = self._obs_client.getObject(bucketName=self._bucket_name, objectKey=object_key)
        if resp.status < 300:
            return io.BufferedReader(resp.body.response)
        else:
            raise DliException(msg="read data from obs failed")

    def next(self):
        pass

    def fetch(self):
        return self._current_row


class ObsReaderIterator(object):
    def __init__(self, obs_reader):
        self._obs_reader = obs_reader

    def __iter__(self):
        return self

    def __next__(self):
        return self.next()

    def next(self):
        try:
            ret = self._obs_reader.next()
            if ret is not True:
                raise StopIteration()
            return self._obs_reader.fetch()
        except IndexError:
            raise StopIteration()


class CsvObsReader(Reader):
    def __init__(self, table_schema, obs_client, bucket_name, obj_prefix):
        super().__init__(table_schema, obs_client, bucket_name, obj_prefix)
        self._csv_reader = None
        self._iter = self._get_next_iter()
        if self._iter is not None:
            self._csv_reader = CsvParser(self._iter)
            self._current_row = None

    def next(self):
        if self._csv_reader is not None:
            try:
                row_data = next(self._csv_reader)
                if row_data:
                    self._get_row(row_data)
                    return True
            except StopIteration:
                # No more data in the current iterator
                pass

            # Cleanup resource
            if self._iter is not None:
                self._iter.close()

            # Check if there are more iterators to process
            if self._next_iter_idx < self._total_iter:
                self._iter = self._get_next_iter()
                if self._iter is not None:
                    self._csv_reader = CsvParser(self._iter)
                    return self.next()
        else:
            logger.info("iter didn't init, create it")
            if self._next_iter_idx < self._total_iter:
                self._iter = self._get_next_iter()
                if self._iter is not None:
                    self._csv_reader = CsvParser(self._iter)
                    return self.next()

        if self._iter is not None:
            self._iter.close()
        return False

    def _get_row(self, row_data):
        sql_cols = self._table_schema.columns
        formatted_row_data = self._update_row(row_data, sql_cols)
        self._current_row = Row(sql_cols, formatted_row_data)

    @staticmethod
    def _update_row(data_list, sql_cols):
        updated_data_list = []
        for col, data in zip(sql_cols, data_list):
            if data is None:
                updated_data_list.append(None)
                continue

            updated_data_list.append(data)
        return updated_data_list


class JsonObsReader(Reader):
    def __init__(self, table_schema, obs_client, bucket_name, obj_prefix):
        super().__init__(table_schema, obs_client, bucket_name, obj_prefix)
        self._iter = self._get_next_iter()
        if self._iter is not None:
            self._next_line = self._iter.readline()

    def next(self):
        if self._iter is not None:
            if len(self._next_line) != 0:
                self._get_row()
                self._next_line = self._iter.readline()
                return True
            else:  # EOF
                self._iter.close()
                if self._next_iter_idx < self._total_iter:
                    self._iter = self._get_next_iter()
                    if self._iter is not None:
                        self._next_line = self._iter.readline()
                        self._get_row()
                        self._next_line = self._iter.readline()
                        return True
        else:
            logger.info("iter didn't init, create it")
            if self._next_iter_idx < self._total_iter:
                self._iter = self._get_next_iter()
                if self._iter is not None:
                    self._next_line = self._iter.readline()
                    self._get_row()
                    self._next_line = self._iter.readline()
                    return True
        if self._iter is not None:
            self._iter.close()
        return False

    def _get_row(self):
        sql_cols = self._table_schema.columns
        row_data = self._update_row(json.loads(self._next_line), sql_cols)
        self._current_row = Row(sql_cols, row_data)

    @staticmethod
    def _update_row(data_dict, sql_cols):
        data_list = []
        for col in sql_cols:
            data = data_dict.get(col.name, None)
            if data is None:
                data_list.append(None)
                continue

            if col.type.startswith("decimal"):
                data_list.append(str(data))
            elif col.type == "timestamp":
                ts_int = time.mktime(datetime.strptime(data, '%Y-%m-%dT%H:%M:%S.%f%z').timetuple())
                ts_int *= 1000
                data_list.append(ts_int)
            else:
                data_list.append(data)
        return data_list
