#!/usr/bin/python
# -*- coding: UTF-8 -*-
import os
import time

from dli.column import Column
from huaweicloudsdkdli.v1 import ShowSqlJobStatusRequest

from dli.dli_client import DliClient
from dli.dli_sql_client import DliSqlClient
from dli.exception import DliException
from dli.sql_engine import SqlEngine
from dli.table_storage_type import TableStorageTypes
from dli.types import DLITypes

PROXY = 'https://ip:port'


def init_aksk_dli_client():
    region = 'cn-north-7'
    project_id = 'xxx'
    ak = os.getenv("HUAWEICLOUD_SDK_AK")
    sk = os.getenv("HUAWEICLOUD_SDK_SK")
    endpoint = 'xxx.myhuaweicloud.com'

    dli_client = DliClient(region=region, project_id=project_id, ak=ak, sk=sk, endpoint=endpoint)
    return dli_client


def init_aksk_dli_sql_client():
    region = 'cn-north-4'
    project_id = 'xxx'
    ak = os.getenv("HUAWEICLOUD_SDK_AK")
    sk = os.getenv("HUAWEICLOUD_SDK_SK")
    endpoint = 'dli.cn-north-4.myhuaweicloud.com'
    obs_endpoint = 'obs.cn-north-4.myhuaweicloud.com'

    dli_client = DliClient(region=region, project_id=project_id, ak=ak, sk=sk, endpoint=endpoint,
                           obs_endpoint=obs_endpoint)
    dli_sql_client = DliSqlClient(dli_client)
    return dli_sql_client


def list_all_sql_jobs(dli_client):
    try:
        sql_jobs = dli_client.list_sql_jobs()
    except DliException as e:
        print(e)
        return
    for sql_job in sql_jobs:
        print(sql_job)


def cancel_sql(dli_client, job_id):
    try:
        dli_client.cancel_sql(job_id)
    except DliException as e:
        print(e)
        return


def del_spark_batch(dli_client, batch_id):
    try:
        resp = dli_client.del_spark_batch_job(batch_id)
        print(resp.msg)
    except DliException as e:
        print(e)
        return


def list_all_queues(dli_client):
    try:
        queues = dli_client.list_queues()
    except DliException as e:
        print(e)
        return

    for queue in queues:
        print(queue.name)


def import_data(dli_client, db_name, tbl_name, queue_name):
    try:
        job_id, status = \
            dli_client.import_table(tbl_name, db_name,
                                    's3a://bucket/obj/data.csv',
                                    'csv', queue_name=queue_name)
    except DliException as e:
        print(e)
        return

    print(job_id)
    print(status)


def export_data(dli_client, db_name, tbl_name, queue_name):
    try:
        job_id, status = dli_client.export_table(tbl_name, db_name,
                                                 's3a://bucket/obj',
                                                 queue_name=queue_name)
    except DliException as e:
        print(e)
        return

    print(job_id)
    print(status)


def run_sql(dli_client, db_name, queue_name):
    # execute SQL
    try:
        sql_job = dli_client.execute_sql('select * from tbl_dli_for_test', db_name, queue_name=queue_name)
        result_set = sql_job.get_result()
    except DliException as e:
        print(e)
        return

    res = dli_client.inner_client.show_sql_job_status(ShowSqlJobStatusRequest(sql_job.job_id))
    if res.result_count == 0:
        return

    for row in result_set:
        print(row)

    # export the query result to obs
    try:
        status = sql_job.export_result('s3a://bucket/obj',
                                       queue_name=queue_name)
    except DliException as e:
        print(e)
        return

    print(status)


def async_run_sql(dli_client, queue_name, db_name):
    try:
        sql_job = dli_client.async_execute_sql('select * from tbl_dli_for_test',
                                               db_name, queue_name)
        while True:
            time.sleep(3)
            job_status = sql_job.get_status()
            if job_status[0] in ['FINISHED', 'CANCELLED', 'FAILED']:
                break

        if job_status[0] != 'FINISHED':
            print('SQL run fail')
            return

        result_set = sql_job.get_result()
        schema = sql_job.get_schema()

    except DliException as e:
        print(e)
        return

    print(schema)

    res = dli_client.inner_client.show_sql_job_status(ShowSqlJobStatusRequest(sql_job.job_id))
    if res.result_count == 0:
        return

    for row in result_set:
        print(row)


def submit_spark_batch_job(dli_client, batch_queue_name, batch_job_info):
    try:
        batch_job = dli_client.submit_spark_batch_job(batch_queue_name, batch_job_info)
    except DliException as e:
        print(e)
        return

    print(batch_job.job_id)
    while True:
        time.sleep(3)
        job_status = batch_job.get_job_status()
        print('Job status: {0}'.format(job_status))
        if job_status == 'dead' or job_status == 'success':
            break

    logs = batch_job.get_driver_log(500)
    for log_line in logs:
        print(log_line)


def list_databases(dli_sql_client, db_pattern, queue, engine="spark", catalog=None):
    try:
        dbs = dli_sql_client.list_databases(database_pattern=db_pattern, queue=queue, engine=engine, catalog=catalog)
        for db in dbs:
            print(db)
    except DliException as e:
        print(e)
        return


def database_exist(dli_sql_client, database, queue, engine="spark"):
    try:
        re = dli_sql_client.database_exists(database=database, queue=queue, engine=engine)
        print(re)
    except DliException as e:
        print(e)
        return


def list_tables(dli_sql_client, database, table_pattern, queue, engine="spark", catalog=None):
    try:
        tbls = dli_sql_client.list_tables(database=database, tbl_pattern=table_pattern, queue=queue, catalog=catalog,
                                          engine=engine)
        for tbl in tbls:
            print(tbl)
    except DliException as e:
        print(e)
        return


def table_exists(dli_sql_client, database, table, queue, engine="spark", catalog=None):
    try:
        re = dli_sql_client.table_exists(database=database, table=table, queue=queue, catalog=catalog, engine=engine)
        print(re)
    except DliException as e:
        print(e)
        return


def get_table(dli_sql_client, database, table, queue, engine="spark", catalog=None):
    try:
        table = dli_sql_client.get_table(database=database, table=table, queue=queue, catalog=catalog, engine=engine)
        print(table.tbl_name)
        print(table.table_type)
    except DliException as e:
        print(e)
        return


def get_table_schema(dli_sql_client, database, table, queue, engine="spark", catalog=None):
    try:
        table = dli_sql_client.get_table_schema(database=database, table=table, queue=queue, catalog=catalog,
                                                engine=engine)
        print(table.table_name)
    except DliException as e:
        print(e)
        return


def create_hive_obs_table(dli_sql_client, database, table, columns, location, storage_type, queue, catalog=None):
    try:
        dli_sql_client.create_hive_obs_table(database=database, table=table, columns=columns, location=location,
                                             storage_type=storage_type, queue=queue, catalog=catalog)
        print("create_hive_obs_table success")
    except DliException as e:
        print(e)
        return


def create_hive_dli_table(dli_sql_client, database, table, columns, storage_type, queue, catalog=None):
    try:
        dli_sql_client.create_hive_dli_table(database=database, table=table, columns=columns,
                                             storage_type=storage_type, queue=queue, catalog=catalog)
        print("create_hive_dli_table success")
    except DliException as e:
        print(e)
        return


def drop_table(dli_sql_client, database, table, queue, catalog=None):
    try:
        dli_sql_client.drop_table(database=database, table=table, queue=queue, catalog=catalog)
        print("drop_table success")
    except DliException as e:
        print(e)
        return


def add_partition(dli_sql_client, database, table, queue, partition, obs_path, catalog=None):
    try:
        dli_sql_client.add_partition(database=database, table=table, queue=queue, partition=partition,
                                     obs_path=obs_path, catalog=catalog)
        print("add_partition success")
    except DliException as e:
        print(e)
        return


def drop_partition(dli_sql_client, database, table, queue, partition, catalog=None):
    try:
        dli_sql_client.drop_partition(database=database, table=table, queue=queue, partition=partition, catalog=catalog)
        print("drop_partition success")
    except DliException as e:
        print(e)
        return


def list_partition(dli_sql_client, database, table, queue, partition_filter=None, catalog=None,
                   engine=SqlEngine.SPARK.value):
    try:
        pts = dli_sql_client.list_partition(database=database, table=table, queue=queue,
                                            partition_filter=partition_filter,
                                            catalog=catalog, engine=engine)
        for item in pts:
            print(item)
    except DliException as e:
        print(e)
        return


def list_functions(dli_sql_client, database, queue, catalog=None, engine=SqlEngine.SPARK.value):
    try:
        functions = dli_sql_client.list_functions(database=database, queue=queue, catalog=catalog, engine=engine)
        for item in functions:
            print(item)
    except DliException as e:
        print(e)
        return


def main():
    dli_client = init_aksk_dli_client()
    dli_sql_client = init_aksk_dli_sql_client()

    # This is an example for show how to use dli python sdk to implement sql application.
    sql_queue_name = 'spark_queue'
    hetu_sql_queue_name = 'hetu_queue'
    tbl_name = 'your_dli_table'
    dli_db_name = 'your_dli_db'
    catalog = 'your_catalog'
    lf_db_name = 'your_catalog_db'
    lf_table_name = 'your_catalog_table'

    dli_tbl_name = 'tbl_dli_for_test'
    obs_tbl_name = 'tbl_obs_for_test'

    list_all_queues(dli_client)

    import_data(dli_client, dli_db_name, dli_tbl_name, sql_queue_name)

    export_data(dli_client, dli_db_name, dli_tbl_name, sql_queue_name)

    run_sql(dli_client, dli_db_name, sql_queue_name)

    async_run_sql(dli_client, sql_queue_name, dli_db_name)

    list_databases(dli_sql_client=dli_sql_client, db_pattern="*a*", queue=sql_queue_name)
    list_databases(dli_sql_client=dli_sql_client, db_pattern="*a*", catalog=catalog, queue=sql_queue_name)
    list_databases(dli_sql_client=dli_sql_client, db_pattern="*a*", catalog=catalog, queue=hetu_sql_queue_name,
                   engine=SqlEngine.HETU.value)
    list_databases(dli_sql_client=dli_sql_client, db_pattern="*a*", queue=hetu_sql_queue_name,
                   engine=SqlEngine.HETU.value)

    database_exist(dli_sql_client=dli_sql_client, database=dli_db_name, queue=sql_queue_name)
    database_exist(dli_sql_client=dli_sql_client, database="fake_tb", queue=sql_queue_name)

    list_tables(dli_sql_client=dli_sql_client, database=dli_db_name, table_pattern="*hive*", queue=sql_queue_name)
    list_tables(dli_sql_client=dli_sql_client, database=dli_db_name, table_pattern="*hive*", queue=hetu_sql_queue_name,
                engine=SqlEngine.HETU.value)
    list_tables(dli_sql_client=dli_sql_client, database=lf_db_name, table_pattern="*a*", catalog=catalog,
                queue=sql_queue_name)
    list_tables(dli_sql_client=dli_sql_client, database=lf_db_name, table_pattern="*a*", catalog=catalog,
                queue=hetu_sql_queue_name, engine=SqlEngine.HETU.value)

    table_exists(dli_sql_client=dli_sql_client, database=dli_db_name, table=tbl_name, queue=sql_queue_name)
    table_exists(dli_sql_client=dli_sql_client, database=dli_db_name, table="fake_table", queue=hetu_sql_queue_name,
                 engine=SqlEngine.HETU.value)

    get_table(dli_sql_client=dli_sql_client, database=dli_db_name, table=tbl_name, queue="fxc_spark_hbo")
    get_table(dli_sql_client=dli_sql_client, database=lf_db_name, table=lf_table_name, queue=sql_queue_name,
              catalog=catalog)
    get_table(dli_sql_client=dli_sql_client, database=dli_db_name, table=tbl_name, queue=hetu_sql_queue_name,
              engine=SqlEngine.HETU.value)
    get_table(dli_sql_client=dli_sql_client, database=lf_db_name, table=lf_table_name, queue=hetu_sql_queue_name,
              catalog=catalog,
              engine=SqlEngine.HETU.value)

    get_table_schema(dli_sql_client=dli_sql_client, database=dli_db_name, table=tbl_name, queue=sql_queue_name)

    create_hive_obs_table(dli_sql_client=dli_sql_client, database=dli_db_name, table="test_new_obs_table",
                          columns=[Column("col1", DLITypes.STRING.value, "comment 1"),
                                   Column("pt2", DLITypes.STRING.value, "", is_partition_column=True)],
                          location="obs://b1/d1", storage_type=TableStorageTypes.PARQUET.value,
                          queue=sql_queue_name)

    create_hive_dli_table(dli_sql_client=dli_sql_client, database=dli_db_name, table="test_new_dli_table",
                          columns=[Column("col1", DLITypes.STRING.value, "comment 1"),
                                   Column("pt2", DLITypes.STRING.value, "", is_partition_column=True)],
                          storage_type=TableStorageTypes.PARQUET.value,
                          queue=sql_queue_name)

    add_partition(dli_sql_client=dli_sql_client, database=dli_db_name, table="tb_partition", partition="pt1='a1'",
                  obs_path="obs://b1/d1", queue=sql_queue_name)

    drop_table(dli_sql_client=dli_sql_client, database=dli_db_name, table="test_new_dli_table", queue=sql_queue_name)

    list_partition(dli_sql_client=dli_sql_client, database=dli_db_name, table="tb_partition",
                   partition_filter="pt1='a1'",
                   queue=sql_queue_name)

    list_partition(dli_sql_client=dli_sql_client, database=dli_db_name, table="tb_partition",
                   partition_filter="pt1='a1'",
                   queue=sql_queue_name)

    list_partition(dli_sql_client=dli_sql_client, database=dli_db_name, table="tb_partition",
                   queue="hetu_sql_queue_name", engine=SqlEngine.HETU.value)

    list_partition(dli_sql_client=dli_sql_client, database=dli_db_name, table="tb_partition", partition_filter="pt1='a1'",
                   queue=hetu_sql_queue_name, engine=SqlEngine.HETU.value)

    list_partition(dli_sql_client=dli_sql_client, database=lf_db_name, table=lf_table_name, catalog=catalog,
                   queue=hetu_sql_queue_name, engine=SqlEngine.HETU.value, partition_filter="pt1='a1'")

    list_functions(dli_sql_client=dli_sql_client, database=dli_db_name, catalog="dli", queue=sql_queue_name,
                   engine=SqlEngine.SPARK.value)

    list_functions(dli_sql_client=dli_sql_client, database=dli_db_name, catalog="dli", queue=hetu_sql_queue_name,
                   engine=SqlEngine.HETU.value)


if __name__ == '__main__':
    main()
